# $Id: ndiary-lib.rb,v 1.11 2001/02/12 15:52:10 not Exp $

NDIARY_VERSION = 'nDiary version 0.9.2'

REQUIRED_RUBY_VER = '1.6'

RUBY_INFO = "ruby #{RUBY_VERSION} (#{RUBY_RELEASE_DATE}) [#{RUBY_PLATFORM}]"

require 'lib/config'
require 'lib/string'
require 'lib/dateformat'
require 'lib/find'
require 'lib/topic'


# tB^
class Filter

  attr_reader :diary

  def initialize(filterlist = [])
    @filterlist = filterlist
    @filterlist.delete_if{ |filter|
      begin
        require 'filter/' + filter + '.rb'
        puts 'load: filter/' + filter + '.rb' if $DEBUG
        false
      rescue LoadError
        $stderr.puts "\n** error: Filter '#{filter}.rb' not found\n"
        true
      end
    }
  end

  def parentdiary=(diary)
    @diary = diary
  end

  def filter(str, type)
    ret = false
    @filterlist.each{ |filter|
      begin
        ret = true if send(filter, str, type) == :THROUGH
      rescue NameError
        $stderr.puts "\n** Filleter error: undefined method '#{filter}'.\n"
        @filterlist.delete(filter)
      end
    }
    return ret
  end

end

# Lt@C쐬̊NX
class Diary
  require 'cgi'

  attr_accessor :logDirectory
  attr_accessor :outputDirectory
  attr_accessor :showVersion
  attr_accessor :format
  attr_accessor :allDiarys
  attr_accessor :months
  attr_accessor :date
  attr_accessor :header
  attr_accessor :topicChar
  attr_accessor :outputDirectory
  attr_accessor :paragraphAnchor 
  attr_accessor :anchorEachTopic
  attr_accessor :monthPartNames
  attr_accessor :lf2br
  attr_accessor :getDateByTimestamp
  attr_accessor :splithtml
  attr_accessor :skelton
  attr_accessor :outputKcode
  attr_accessor :filter

  # 
  def initialize
    @diarys          = []
    @showVersion     = true
    @logDirectory    = ''
    @outputDirectory = ''
    @allDiarys       = {}
    @months          = []
    @format          = '%b.%d,%Y (%a)'
    @header          = ''
    @lf2br           = true
    @outputKcode     = 1
    @now             = Time::now
    @filter = nil
  end

  def logDirectory=(dir)
    dir.gsub!(/~/, ENV['HOME'].to_s)
    dir << '/' unless dir[-1,1] == '/'
    unless File::directory?(dir) then
      $stderr.puts "\n** error: '#{dir}' not exist.\n"
      exit
    end
    @logDirectory = dir
    if @logDirectory == @outputDirectory then
      $stderr.puts "\n** error: LOG_DIRECTORY and OUTPUT_DIRECTORY should not be the same directory.\n"
      exit
    end      
  end

  def outputDirectory=(dir)
    dir.gsub!(/~/, ENV['HOME'].to_s)
    dir << '/' unless dir[-1,1] == '/'
    unless File::directory?(dir) then
      $stderr.puts "\n** error: '#{dir}' not exist.\n"
      exit
    end
    @outputDirectory = dir
    if @logDirectory == @outputDirectory then
      $stderr.puts "\n** error: LOG_DIRECTORY and OUTPUT_DIRECTORY should not be the same directory.\n"
      exit
    end      
  end

  # Lt@C擾
  def getDiarys(diary = nil)
    if diary.kind_of?(Diary) then
      @allDiarys.update(diary.allDiarys)
      @months = diary.months
    else
      Dir::find(@logDirectory){ |file|
        next unless /\/(\d{8})(\.diary|\.dia)$/i =~ file
        date = File::basename(file, $2)
        unless @allDiarys.key?(date) then
          @allDiarys[date] = file
        else
          $stderr.puts "\n** error: #{date}'s diary duplicated\n"
          exit
        end
      }

      @allDiarys.keys.each{ |date|
        @months << date[0..5]
      }
      Dir[@outputDirectory + '[0-9]' * 6 + '.html'].each{ |date|
        @months << File::basename(date)[0..5]
      }
      @months = @months.uniq.sort
    end
  end

  # ro
  def makeFootNote(dstr)
    str = dstr.dup
    str.gsub!(/\([\*]([^\)]+)\)|i[\*]([^j]+)j|\(\(-(.+?)-\)\)/){
      @cntFootNote += 1
      nameFootNote = "#{@date}_#{@cntFootNote}"
      @strFootNote << "\t\t<a name=\"#{nameFootNote}\" href=\"#{@yyyy}#{@mm}#{@monthPartNames[@dd.to_i]}.html##{nameFootNote}\">\*#{@cntFootNote}</a>: #{$+}<br>\n"
      "<sup><small>(<a href=\"##{nameFootNote}\">*#{@cntFootNote}</a>\)</small></sup>"
    }
    str.gsub!(/\(\\\*/,"\(\*")
    return str
  end

  private

  # ȉ
  def makeParagraph(str)
    return str if @filter.filter(str, :PRE)
    
    # C
    if str[0,3] == "<<\n" then
      return str if @filter.filter(str, :HEREDOC)
      str = str[3...str.size]
      str.gsub!("\n","\n\t")
      str = "\t#{str}\n\n"
      @filter.filter(str, :AFTER_HEREDOC)
      return str

    # R[h
    elsif /\ACode:(?: (-.*))?\n/i =~ str then
      str = $'
      param = $1
      return str if @filter.filter(str, :CODE)
      t = /t(\d+)/i =~ param ? $1.to_i : 4
      str = CGI::escapeHTML(str) if /q/i =~ param
      str = str.expandtabs(t)
      str = "\t<div class=\"code\"><pre><tt><code>\n#{str}</code></tt></pre></div>\n\n"
      @filter.filter(str, :AFTER_CODE)
      return str

    # Quote
    elsif /\AQuote:(?: (-.*))?\n/i =~ str then
      str = $'
      param = $1
      return str if @filter.filter(str, :QUOTE)
      t = /t(\d+)/i =~ param ? $1.to_i : 4
      str = CGI::escapeHTML(str) if /q/i =~ param
      if /l[^-]/i =~ param or (@lf2br != false and /l-/i !~ param) then
        str.gsub!("\n","<br>\n")
      end
      str = str.expandtabs(t)
      str = "\t<blockquote class=\"quote\">\n#{str}</blockquote>\n\n"
      @filter.filter(str, :AFTER_QUOTE)
      return str

    # p <blockquote>
    elsif /^[^\t]/ !~ str then
      return str if @filter.filter(str, :BLOCKQUOTE)
      str.delete!("\t")
      str.gsub!("\n","<br>\n\t\t") unless @lf2br == false
      str = makeFootNote(str)
      str = "\t<blockquote>\n\t\t#{str}\n\t</blockquote>\n\n"
      @filter.filter(str, :AFTER_BLOCKQUOTE)
      return str

    # `Xg <dl>
    elsif /^\t/ =~ str then
      return str if @filter.filter(str, :DL)
      str.gsub!(/^/,"\t\t<dt>")
      str.gsub!(/^\t\t<dt>\t/,"\t\t\t<dd>")
      str.gsub!(/^(\t\t<dt>.+)$/,"\\1</dt>")
      str.gsub!(/^(\t\t\t<dd>.+)$/,"\\1</dd>")
      str = makeFootNote(str)
      str = "\t<dl>\n#{str}\n\t</dl>\n\n"
      @filter.filter(str, :AFTER_DL)
      return str

    # o <h5>
    elsif @topicChar.index((th, ts = str.split(//,2))[0]
    ) and (/\n/ !~ str) then
      return str if @filter.filter(ts, :TOPIC)
      @cntTopic += 1
      @cntParagraph = 0 if @anchorEachTopic == true
      if self.kind_of?(PastDiary) then
        @topics[@dd] = [] unless @topics.key?(@dd)
        @topics[@dd] << th + ts
      end
      
      str = "\t<h5><a name=\"#{@dd}_t#{@cntTopic}\" href=\"#{@yyyy}#{@mm}#{@monthPartNames[@dd.to_i]}.html##{@dd}_t#{@cntTopic}\">#{th}</a>#{ts}</h5>\n\n"
      @filter.filter(str, :AFTER_TOPIC)
      return str

    # sXg <ul>
    elsif /\A[\+E]/ =~ str then
      return str if @filter.filter(str, :UL)
      str.gsub!(/\n([^\+E].+)$/,"<br>\\1") unless @lf2br == false
      str.gsub!(/(?:\n|^)[\+E](.+)(?!^[\+E])/,"\n\t\t<li>\\1</li>")
      str = makeFootNote(str)
      str = "\t<ul>#{str}\n\t</ul>\n\n"
      @filter.filter(str, :AFTER_UL)
      return str

    # ʏi <p>
    elsif !str.strip.empty? then
      return str if @filter.filter(str, :P)
      @cntParagraph += 1
      str.gsub!("\n","<br>\n\t") unless @lf2br == false
      str = makeFootNote(str)
      if @paragraphAnchor then
        anchor = ''
        if @anchorEachTopic then
          anchor = "#{@dd}_t#{@cntTopic}_#{@cntParagraph}"
        else
          anchor = "#{@dd}_#{@cntParagraph}"
        end
        str = "\t<p><a name=\"#{anchor}\" href=\"#{@yyyy}#{@mm}#{@monthPartNames[@dd.to_i]}.html##{anchor}\">#{@header}</a>\n\t#{str}</p>\n\n"
      else
        str = "\t<p>#{str}</p>\n\n"
      end
      @filter.filter(str, :AFTER_P)
      return str

    # 󕶎
    else 
      return ''
    end
  end

  # tƂ html 쐬
  def tohtml(date)
    body = ''
    @cntParagraph = @cntTopic = @cntFootNote = 0
    @strFootNote = ''
    filename = @allDiarys[date]
    
    # tߍ
    @date = date
    @yyyy = @date[0..3]
    @mm   = @date[4..5]
    @dd   = @date[6..7]

    objDate = Time::local(@yyyy, @mm, @dd)

    date_anchor = objDate.strftime2("<h4><a name=\"#{@dd}\" href=\"#{@yyyy}#{@mm}#{@monthPartNames[@dd.to_i]}.html##{@dd}\">#{@format}</a></h4>\n")
    @filter.filter(date_anchor, :DATE_ANCHOR)
    body << date_anchor
    body << "<div class=\"day\">\n\n"



    # Read Diary File
    text = ''
    open(filename){ |file| text = file.read.gsub(/\r\n/,"\n") }
    text = text.tosjis

    # tB^ :ONEDAY
    if @filter.filter(text, :ONEDAY) then
      body << text
    else
      text.each("\n\n"){ |line|
        line.sub!(/^\n$/,"")
        line.chomp!
        line.chomp!
        line.delete!("\C-z")
        next if line.empty?
        body << makeParagraph(line)
      }
    end

    # rǉ
    unless @strFootNote.empty? then
      body << "\t<div class=\"footnote\"><hr>\n#{@strFootNote}\n\t</div>\n"
    end

    # ǉ
    splithtml = objDate.strftime2(@splithtml).gsub(/\"/, '\"')
    splithtml = eval('"' + splithtml + '"')

    body << "</div>\n\n"
    body << splithtml + "\n\n" unless @splithtml.empty?

    # tB^ :AFTER_ONEDAY
    @filter.filter(text, :AFTER_ONEDAY)

    return body
  end

  # htmlt@C쐬
  def writeHTML(filename)
    @outputKcode = 1 if (@outputKcode < 1 or @outputKcode > 3)
    begin

      # L{
      body = ''
      @diarys.each{ |date|
        body << tohtml(date).kconv(@outputKcode)
      }

      out = open(filename, "w")

      infile = open(@logDirectory + @skelton)
      infile.each { |line|

        # Diary Insert
        if line.gsub!(/<!--body-->/i, '') then

          # version \
          if @showVersion then
            out.print "\n<div class=\"version\">Generated by ",
                      "<a href=\"http://www14.cds.ne.jp/~not/product/ndiary/\">",
                       NDIARY_VERSION, "</a></div>\n<hr>\n"
          else
            out.print "\n<!--Generated by #{NDIARY_VERSION}-->\n"
          end
          
          # Last-modified ̖ߍ݁i߉Hj
          lm = @now.dup.gmtime
          out.puts lm.strftime("<!--Last-Modified :%a, %d %b %Y %H:%M:%S GMT-->")

          # L}
          out.print body
          
          # ruby version }
          out.print "<!--#{RUBY_INFO}-->\n\n"

        end

        # Load Plug-in
        line.gsub!(/<!--plug=(.+)-->/i){
          argv = []
          argv = $1.gsub('&gt;','>').gsub('&lt;','<').split(/\?/s)
          plugname = argv.shift
          plugname = plugname + '.rb'

          opt = {}
          argv.each{ |str|
            str = str.gsub('&question;','?').gsub('&amp;','&')
            key, val = str.split(/=/, 2)
            opt[key.downcase] = val
          }

          if $DEBUG then
            load 'plugin/' + plugname
            plug(opt)
          else
            begin
              load 'plugin/' + plugname
              plug(opt)
            rescue
              $stderr.puts "\n** Plug-in error: #{plugname}\n"
              "<!--plug-in error: #{plugname}-->"
            end
          end
        }

        out.print line.kconv(@outputKcode)
      }
      infile.close

      out.close

      print "make '#{filename}'\n"

    rescue Errno::ENOENT
      $stderr.puts "\n** error: skelton file '#{@logDirectory}#{@skelton}' not found\n"
      exit
    end
  end

end


# ŋ߂̓L̃_CWFXg쐬NX
class LastDiary < Diary

  attr_accessor :filename
  attr_accessor :maxDiarys

  # 
  def initialize
    super()
    @maxDiarys = 5
    @filename  = 'index.html'
  end

  # tƂ html 쐬
  def tohtml(date)
    body = super
    if date == @diarys[0] then
      body = "\n<!-- PiloWeb_Begin -->\n#{body}<!-- PiloWeb_End -->\n\n"
    end
    return body
  end

    # Lt@C쐬
  def makeHTML
    @diarys = @allDiarys.keys.sort.reverse[0...@maxDiarys]
    lastDate = @diarys[0]

    @objLastDate = Time::local(lastDate[0..3], lastDate[4..5], lastDate[6..7])

    # Write html file
    filename = @outputDirectory + @filename
    writeHTML(filename)
  end

end

# ʃt@C쐬NX
class PastDiary < Diary

  # 
  def initialize
    super()
    @targetMonth = ''
    @topics      = {}
  end

  def targetMonth=(month)
    @topics.clear
    @targetMonth = month
  end

  # Lt@C쐬
  def makeHTML
    diarys = @allDiarys.keys.sort
    diarys.delete_if{ |file|
      file.index(@targetMonth) == nil
    }

    if diarys.empty? then
      $stderr.print @targetMonth + ": not found  .diary files\n"
      return
    end
    @objTargetMonth = Time::local(@targetMonth[0..3], @targetMonth[4..5])

    # Write html file
    partdiarys = {}
    diarys.each{ |date|
      day = date[6..7].to_i
      unless partdiarys.key?(@monthPartNames[day]) then
        partdiarys[@monthPartNames[day]] = []
      end
      partdiarys[@monthPartNames[day]] << date
    }

    partdiarys.each{|part, ds|
      filename = @outputDirectory + @targetMonth + part.to_s + ".html"
      @diarys = ds.sort
      writeHTML(filename)
    }

    # Topics List
    open(@logDirectory + @targetMonth + '.topic', "w"){ |file|
      @topics.keys.sort.each{ |dd|
        file.puts "#@targetMonth#{dd}\t#{@topics[dd].join(\"\t\")}"
      }
    } unless @topics.empty?
  end
end



#///////////////////////////////////////////////////////////////////
#
# Main Routine                                                      
#

def main

  ti = Time::now
  print "\n-- #{NDIARY_VERSION} --\n\n"

  if RUBY_VERSION < REQUIRED_RUBY_VER then
    $stderr.puts "\n** error: this script require ruby #{REQUIRED_RUBY_VER} or later.\n"
    exit
  end

  # Load Configuration
  if /.+\.conf$/i =~ ARGV[0] then
    configfile = ARGV.shift
  else
    configfile = ENV['HOME'].to_s + File::Separator + ".ndiaryrc";
    unless File::readable? configfile then
      configfile = 'ndiary.conf'
    end
  end

  unless File::readable? configfile then
    $stderr.puts "\n** error: config file '#{configfile} not found.\n"
    exit
  end
  
  config = Config::new
  config.setarray('TOPIC', 'FILTER')
  config.load(configfile)

  filter = Filter::new(config['FILTER'])
  config['MONTH_PART_NAMES'] = [] unless config['MONTH_PART_NAMES'].kind_of?(Array)
  monthPartNames = [''] * 32
  monthPartNames[1,0] = config['MONTH_PART_NAMES']
  monthPartNames = monthPartNames[0..31]

  # ʂ̓L쐬̉
  pastDiary                    = PastDiary::new
  pastDiary.logDirectory       = config['LOG_DIRECTORY']
  pastDiary.showVersion        = config['SHOW_VERSION']
  pastDiary.outputDirectory    = config['OUTPUT_DIRECTORY']
  pastDiary.outputKcode        = config['OUTPUT_KCODE']
  pastDiary.format             = config['DATEFORMAT_PASTDAYS']
  pastDiary.header             = config['HEADER_PASTDAYS']
  pastDiary.topicChar          = config['TOPIC_CHAR']
  pastDiary.paragraphAnchor    = config['PARAGRAPH_ANCHOR']
  pastDiary.anchorEachTopic    = config['ANCHOR_EACH_TOPIC']
  pastDiary.lf2br              = config['LF2BR']
  pastDiary.skelton            = config['SKELTON_PASTDAYS']
  pastDiary.splithtml          = config['SPLIT_PASTDAYS']
  pastDiary.monthPartNames     = monthPartNames
  pastDiary.filter             = filter
  pastDiary.filter.parentdiary = pastDiary
  pastDiary.getDiarys


   if pastDiary.allDiarys.keys.nil? or pastDiary.allDiarys.keys.size < 1 then
      $stderr.puts "\n** error: no .diary file under #{pastDiary.logDirectory}.\n"
      exit(1)
   end

  # ŋ߂̓L̃_CWFXg쐬
  if ARGV.size.zero? then
    pastDiary.targetMonth = pastDiary.allDiarys.keys.sort.reverse[0][0..5]
    pastDiary.makeHTML

    lastDiary                    = LastDiary::new
    lastDiary.maxDiarys          = config['MAX_DIARY']
    lastDiary.showVersion        = config['SHOW_VERSION']
    lastDiary.logDirectory       = config['LOG_DIRECTORY']
    lastDiary.outputDirectory    = config['OUTPUT_DIRECTORY']
    lastDiary.outputKcode        = config['OUTPUT_KCODE']
    lastDiary.format             = config['DATEFORMAT_LASTEST']
    lastDiary.header             = config['HEADER_LASTEST']
    lastDiary.topicChar          = config['TOPIC_CHAR']
    lastDiary.paragraphAnchor    = config['PARAGRAPH_ANCHOR']
    lastDiary.anchorEachTopic    = config['ANCHOR_EACH_TOPIC']
    lastDiary.lf2br              = config['LF2BR']
    lastDiary.skelton            = config['SKELTON_LASTEST']
    lastDiary.splithtml          = config['SPLIT_LASTEST']
    lastDiary.filename           = config['LASTEST_FILENAME']
    lastDiary.monthPartNames     = monthPartNames
    lastDiary.getDiarys(pastDiary)
    lastDiary.filter             = filter
    lastDiary.filter.parentdiary = lastDiary
    lastDiary.makeHTML

  # Nw肵č쐬
  elsif ARGV[0] =~ /^(\d{6})$/ then
    pastDiary.targetMonth = $1
    pastDiary.makeHTML

  # SLt@Cč쐬
  elsif ARGV[0].downcase == 'all' then
    for month in pastDiary.months
      pastDiary.targetMonth = month
      pastDiary.makeHTML
    end

  # sȃp[^
  else
    $stderr.puts "\n** Bad Parameter: '#{ARGV[0]}'\n"
    exit
  end

  # gsbNꗗ쐬
  if config['TOPIC'].size > 0 then
    t = Topics::new
    t.logDirectory     = config['LOG_DIRECTORY']
    t.outputDirectory  = config['OUTPUT_DIRECTORY']
    t.topicDateFormat  = config['TOPIC_DATE_FORMAT']
    t.topicMonthFormat = config['TOPIC_MONTH_FORMAT']
    t.monthPartNames   = monthPartNames
    t.outputKcode      = config['OUTPUT_KCODE']
    config['TOPIC'].each{ |topic|
      filename, regexp, pattern, replace = topic.split(/\t/)
      t.filename = filename
      t.regexp  = regexp  ? regexp  : ''
      t.pattern = pattern ? pattern : ''
      t.replace = replace ? replace : ''
      t.writeHTML(pastDiary.allDiarys.keys)
    }
  end

  puts "\nfinish. (#{Time::now - ti}sec.)"
end
